# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/nb_helpers/',
                'doc_host': 'https://wandb.github.io',
                'git_url': 'https://github.com/wandb/nb_helpers/tree/main/',
                'lib_path': 'nb_helpers'},
  'syms': { 'nb_helpers.actions': { 'nb_helpers.actions.create_comment_body': ('actions.html#create_comment_body', 'nb_helpers/actions.py'),
                                    'nb_helpers.actions.create_issue_nb_fail': ( 'actions.html#create_issue_nb_fail',
                                                                                 'nb_helpers/actions.py'),
                                    'nb_helpers.actions.download': ('actions.html#download', 'nb_helpers/actions.py'),
                                    'nb_helpers.actions.download_payload': ('actions.html#download_payload', 'nb_helpers/actions.py'),
                                    'nb_helpers.actions.get_api': ('actions.html#get_api', 'nb_helpers/actions.py'),
                                    'nb_helpers.actions.get_colab_url2md': ('actions.html#get_colab_url2md', 'nb_helpers/actions.py'),
                                    'nb_helpers.actions.open_zip': ('actions.html#open_zip', 'nb_helpers/actions.py'),
                                    'nb_helpers.actions.post_colab_links': ('actions.html#post_colab_links', 'nb_helpers/actions.py'),
                                    'nb_helpers.actions.update_comment': ('actions.html#update_comment', 'nb_helpers/actions.py'),
                                    'nb_helpers.actions.upload_at': ('actions.html#upload_at', 'nb_helpers/actions.py'),
                                    'nb_helpers.actions.upload_modified_nbs': ( 'actions.html#upload_modified_nbs',
                                                                                'nb_helpers/actions.py')},
            'nb_helpers.clean': { 'nb_helpers.clean._create_table': ('clean.html#_create_table', 'nb_helpers/clean.py'),
                                  'nb_helpers.clean.clean_all': ('clean.html#clean_all', 'nb_helpers/clean.py'),
                                  'nb_helpers.clean.clean_cell': ('clean.html#clean_cell', 'nb_helpers/clean.py'),
                                  'nb_helpers.clean.clean_cell_output': ('clean.html#clean_cell_output', 'nb_helpers/clean.py'),
                                  'nb_helpers.clean.clean_nb': ('clean.html#clean_nb', 'nb_helpers/clean.py'),
                                  'nb_helpers.clean.clean_nbs': ('clean.html#clean_nbs', 'nb_helpers/clean.py'),
                                  'nb_helpers.clean.clean_one': ('clean.html#clean_one', 'nb_helpers/clean.py'),
                                  'nb_helpers.clean.clean_output_data_vnd': ('clean.html#clean_output_data_vnd', 'nb_helpers/clean.py'),
                                  'nb_helpers.clean.rm_execution_count': ('clean.html#rm_execution_count', 'nb_helpers/clean.py')},
            'nb_helpers.colab': { 'nb_helpers.colab._create_colab_cell': ('colab.html#_create_colab_cell', 'nb_helpers/colab.py'),
                                  'nb_helpers.colab.add_colab_badge': ('colab.html#add_colab_badge', 'nb_helpers/colab.py'),
                                  'nb_helpers.colab.add_colab_metadata': ('colab.html#add_colab_metadata', 'nb_helpers/colab.py'),
                                  'nb_helpers.colab.create_colab_badge_cell': ('colab.html#create_colab_badge_cell', 'nb_helpers/colab.py'),
                                  'nb_helpers.colab.get_colab_url': ('colab.html#get_colab_url', 'nb_helpers/colab.py'),
                                  'nb_helpers.colab.has_colab_badge': ('colab.html#has_colab_badge', 'nb_helpers/colab.py')},
            'nb_helpers.docker': {},
            'nb_helpers.export': { 'nb_helpers.export._export_code_cell': ('export.html#_export_code_cell', 'nb_helpers/export.py'),
                                   'nb_helpers.export._export_md_cell': ('export.html#_export_md_cell', 'nb_helpers/export.py'),
                                   'nb_helpers.export._export_one': ('export.html#_export_one', 'nb_helpers/export.py'),
                                   'nb_helpers.export.colab_url': ('export.html#colab_url', 'nb_helpers/export.py'),
                                   'nb_helpers.export.default_line_filter': ('export.html#default_line_filter', 'nb_helpers/export.py'),
                                   'nb_helpers.export.export': ('export.html#export', 'nb_helpers/export.py'),
                                   'nb_helpers.export.export_cell': ('export.html#export_cell', 'nb_helpers/export.py'),
                                   'nb_helpers.export.export_nbs': ('export.html#export_nbs', 'nb_helpers/export.py'),
                                   'nb_helpers.export.filter_out_lines': ('export.html#filter_out_lines', 'nb_helpers/export.py'),
                                   'nb_helpers.export.read_list_file': ('export.html#read_list_file', 'nb_helpers/export.py')},
            'nb_helpers.run': { 'nb_helpers.run.CaptureShell.prettytb': ('run.html#captureshell.prettytb', 'nb_helpers/run.py'),
                                'nb_helpers.run.exec_nb': ('run.html#exec_nb', 'nb_helpers/run.py'),
                                'nb_helpers.run.run_nbs': ('run.html#run_nbs', 'nb_helpers/run.py'),
                                'nb_helpers.run.run_one': ('run.html#run_one', 'nb_helpers/run.py'),
                                'nb_helpers.run.skip_nb': ('run.html#skip_nb', 'nb_helpers/run.py')},
            'nb_helpers.utils': { 'nb_helpers.utils.RichLogger': ('utils.html#richlogger', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger.__init__': ('utils.html#richlogger.__init__', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger._format_colab_link': ( 'utils.html#richlogger._format_colab_link',
                                                                                      'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger._format_colab_link_md': ( 'utils.html#richlogger._format_colab_link_md',
                                                                                         'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger.error': ('utils.html#richlogger.error', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger.exception': ('utils.html#richlogger.exception', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger.info': ('utils.html#richlogger.info', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger.to_csv': ('utils.html#richlogger.to_csv', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger.to_md': ('utils.html#richlogger.to_md', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger.to_table': ('utils.html#richlogger.to_table', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger.warning': ('utils.html#richlogger.warning', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger.writerow': ('utils.html#richlogger.writerow', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.RichLogger.writerow_incolor': ( 'utils.html#richlogger.writerow_incolor',
                                                                                    'nb_helpers/utils.py'),
                                  'nb_helpers.utils._csv_to_md': ('utils.html#_csv_to_md', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils._format_row': ('utils.html#_format_row', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils._get_github_repo_remote': ('utils.html#_get_github_repo_remote', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.create_table': ('utils.html#create_table', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.csv_to_md': ('utils.html#csv_to_md', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.detect_imported_libs': ('utils.html#detect_imported_libs', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.extract_libs': ('utils.html#extract_libs', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.find_nbs': ('utils.html#find_nbs', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.get_repo': ('utils.html#get_repo', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.git_branches': ('utils.html#git_branches', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.git_current_branch': ('utils.html#git_current_branch', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.git_last_commit': ('utils.html#git_last_commit', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.git_local_repo': ('utils.html#git_local_repo', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.git_main_name': ('utils.html#git_main_name', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.git_origin_repo': ('utils.html#git_origin_repo', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.is_nb': ('utils.html#is_nb', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.print_output': ('utils.html#print_output', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.remove_rich_format': ('utils.html#remove_rich_format', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.search_cell': ('utils.html#search_cell', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.search_cells': ('utils.html#search_cells', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.search_string_in_nb': ('utils.html#search_string_in_nb', 'nb_helpers/utils.py'),
                                  'nb_helpers.utils.today': ('utils.html#today', 'nb_helpers/utils.py')},
            'nb_helpers.wandb': { 'nb_helpers.wandb.filter_nbs': ('wandb.html#filter_nbs', 'nb_helpers/wandb.py'),
                                  'nb_helpers.wandb.fix_nbs': ('wandb.html#fix_nbs', 'nb_helpers/wandb.py'),
                                  'nb_helpers.wandb.get_wandb_tracker': ('wandb.html#get_wandb_tracker', 'nb_helpers/wandb.py'),
                                  'nb_helpers.wandb.search_code': ('wandb.html#search_code', 'nb_helpers/wandb.py'),
                                  'nb_helpers.wandb.summary_nbs': ('wandb.html#summary_nbs', 'nb_helpers/wandb.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/06_export.ipynb.

# %% auto 0
__all__ = ['default_line_filter', 'filter_out_lines', 'export_cell', 'colab_url', 'export', 'read_list_file', 'export_nbs']

# %% ../nbs/06_export.ipynb 3
import os
from pathlib import Path
from rich.markdown import Markdown
from fastcore.script import call_parse, Param, store_true

from .colab import has_colab_badge, get_colab_url
from .utils import find_nbs, read_nb, is_nb

# %% ../nbs/06_export.ipynb 8
def _export_code_cell(source):
    "Wrap code around python"
    return f"\n```python\n{source}\n```\n\n"

def default_line_filter(line):
    if "colab.research" in line:
        return False
    elif "@wandbcode" in line:
        return False
    else: 
        return True

def filter_out_lines(source, f=default_line_filter):
    "Filter lines from source with filter f"
    lines = source.split("\n")
    return "\n".join(filter(f, lines))

def _export_md_cell(source):
    source = filter_out_lines(source)
    return f"{source} \n"

# %% ../nbs/06_export.ipynb 9
def export_cell(cell, debug=False):
    "Export cell source to string"
    source = cell["source"]
    if debug:
        print(source)
    if cell["cell_type"] == "markdown":
        return _export_md_cell(source)
        
    elif cell["cell_type"] == "code":
        return  _export_code_cell(source)
    else:
        return ""

# %% ../nbs/06_export.ipynb 12
def colab_url(file):
    "Create a fresh colab URL from file, must be on github repo"
    url = get_colab_url(file, branch="master")
    return f'\n[![](https://colab.research.google.com/assets/colab-badge.svg)]({url})\n\n'

# %% ../nbs/06_export.ipynb 17
def _export_one(path, outfile, verbose=False):
    
    notebook = read_nb(path)
    
    cells = notebook["cells"]
    
    exported_cells = []
    for cell in cells:
        md_cell = export_cell(cell)
        source = cell["source"]
        if verbose:
            print(f"=================\nInput: \n {source}")
            print(f"-----------------\nMD: \n {md_cell}")
        exported_cells.append(md_cell)
    
    with open(outfile, "w") as f:
        f.writelines(exported_cells)

# %% ../nbs/06_export.ipynb 18
@call_parse
def export(
    path: Param("A path to nb files", Path, nargs="?", opt=False) = os.getcwd(),
    verbose: Param("Print errors along the way", store_true) = False,
    outfile: Param("An output file to save") = None,
):
    if not is_nb(path):
        raise Exception(f"This {path} is not a notebook!")
    
    if outfile is None:
        outfile = path.with_suffix(".md")
        print(f"Exporting notebook {path} -> {outfile}")
    
    _export_one(path, outfile, verbose)

# %% ../nbs/06_export.ipynb 20
def read_list_file(fname):
    "Reads a file into a list"
    with open(fname, "r") as f:
        lines = f.readlines()
        
    return [Path(l.replace("\n", "")) for l in lines if l is not ""]

# %% ../nbs/06_export.ipynb 21
@call_parse
def export_nbs(
    file: Param("A text file containing a list of notebooks to export", Path),
    verbose: Param("Print errors along the way", store_true) = False,
    outpath: Param("An output folder to save the exported notebooks", Path) = Path(".") ,
):
    nb_paths = read_list_file(file)
    
    for nb_path in nb_paths:
        if not is_nb(nb_path):
            raise Exception(f"This {nb_path} is not a notebook!")
        outfile = outpath/(nb_path.with_suffix(".md").name)
        print(f"Exporting notebook {nb_path} -> {outfile}")
        _export_one(nb_path, outfile, verbose=verbose)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_colab.ipynb.

# %% auto 0
__all__ = ['get_colab_url', 'has_colab_badge', 'create_colab_badge_cell', 'add_colab_badge', 'add_colab_metadata']

# %% ../nbs/00_colab.ipynb 3
from pathlib import Path
from IPython import get_ipython
from IPython.display import display, Markdown

from fastcore.basics import ifnone

from execnb.nbio import NbCell, read_nb

from .utils import git_main_name, git_origin_repo, git_local_repo, search_cell, git_current_branch

# %% ../nbs/00_colab.ipynb 7
def get_colab_url(fname, branch):
    "Get git repo url, to append to colab"
    fname = Path(fname).resolve()
    github_repo = git_origin_repo(fname)
    fname = fname.relative_to(git_local_repo(fname))

    return f"https://colab.research.google.com/github/{github_repo}/blob/{branch}/{str(fname)}"

# %% ../nbs/00_colab.ipynb 9
_badge_meta = {"id": "view-in-github", "colab_type": "text"}

# %% ../nbs/00_colab.ipynb 10
def _create_colab_cell(url, meta={}, tracker=None):
    "Creates a notebook cell with the `Open In Colab` badge"
    tracker = ifnone(tracker, "")
    data = {
        "cell_type": "markdown",
        "metadata": meta,
        "source": f'<a href="{url}" target="_parent"><img src="https://colab.research.google.com/assets/colab-badge.svg" alt="Open In Colab"/></a>\n'
        + tracker,
    }
    return NbCell(-1, data)

# %% ../nbs/00_colab.ipynb 12
def has_colab_badge(nb):
    "Check if notebook has colab badge, returns the cell position, -1 if not present"
    for i, cell in enumerate(nb["cells"]):
        if search_cell(cell, "Open In Colab"):
            return i
    return -1

# %% ../nbs/00_colab.ipynb 14
def create_colab_badge_cell(fname, branch=None, meta={}, tracker=None):
    "Create a colab badge cell from `fname`"
    # get main/master name
    branch = ifnone(branch, git_main_name(fname))
    url = get_colab_url(fname, branch)
    colab_cell = _create_colab_cell(url, meta, tracker)
    return colab_cell

# %% ../nbs/00_colab.ipynb 16
def add_colab_badge(notebook, fname, branch=None, idx=0, meta=_badge_meta, tracker=None):
    "Add a badge to Open In Colab in the `idx` cell"
    idx_colab_badge = has_colab_badge(notebook)
    if idx_colab_badge != -1:
        notebook["cells"].pop(idx_colab_badge)
    colab_cell = create_colab_badge_cell(fname, branch, meta, tracker)
    notebook["cells"].insert(idx, colab_cell)
    return notebook

# %% ../nbs/00_colab.ipynb 23
_colab_meta = {
    "colab": {
        "include_colab_link": True, 
        "toc_visible": True,
        "provenance": []
        },
    "kernelspec": {
      "name": "python3",
      "display_name": "Python 3"
      },
    "language_info": {
      "name": "python"
      },
    "accelerator": "GPU",
    "gpuClass": "standard"
    }


# %% ../nbs/00_colab.ipynb 24
def add_colab_metadata(notebook, meta=_colab_meta):
    "Adds GPU and colab meta to `notebook`"
    if "accelerator" in notebook: # old colaboratory standard
        del notebook["accelerator"]
    notebook["metadata"].update(_colab_meta)
    return notebook



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/05_run.ipynb.

# %% auto 0
__all__ = ['logger', 'skip_nb', 'exec_nb', 'run_one', 'run_nbs']

# %% ../nbs/05_run.ipynb 3
import time, os, logging, re
from pathlib import Path
from typing import Union

from fastcore.basics import patch
from fastcore.script import call_parse, Param, store_true
from rich.progress import Progress

from execnb.nbio import read_nb
from execnb.shell import *
from .actions import create_issue_nb_fail

from .utils import find_nbs, git_main_name, search_string_in_nb, RichLogger
from .colab import get_colab_url

# %% ../nbs/05_run.ipynb 5
logger = RichLogger(columns=["fname", "status", "t[s]"])

# %% ../nbs/05_run.ipynb 6
def skip_nb(notebook, filters=None):
    "check for notebook filters: tensorflow, pytorch, ..."
    if filters is None: 
        return False
    return search_string_in_nb(notebook, filters)

# %% ../nbs/05_run.ipynb 8
def exec_nb(fname, pip_install=True):
    "Execute tests in notebook in `fn`"
    nb = read_nb(fname)

    def preproc(cell):
        logger.info(cell.source)
        if (cell.cell_type == "code" and "!pip install" in cell.source and not pip_install) :
            return True
        if cell.cell_type != "code":
            return True
        else:
            return False

    shell = CaptureShell(fname)
    try:
        shell.run_all(nb, exc_stop=True, preproc=preproc)
    except Exception as e:
        return False, shell
    return True, shell

# %% ../nbs/05_run.ipynb 10
@patch
def prettytb(
    self: CaptureShell, fname: Union[Path, str] = None, simple=False
):  # filename to print alongside the traceback
    "Show a pretty traceback for notebooks, optionally printing `fname`."
    fname = fname if fname else self._fname
    _fence = "=" * 75
    cell_intro_str = f"While Executing Cell #{self._cell_idx}:" if self._cell_idx else "While Executing:"
    cell_str = f"\n{cell_intro_str}\n{self.exc[-1]}"
    fname_str = f" in {fname}" if fname else ""
    res = f"{type(self.exc[1]).__name__}{fname_str}:\n{_fence}\n{cell_str}\n"
    if simple:
        ansi_escape = re.compile(r"\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])")
        res = ansi_escape.sub("", res)
    return res

# %% ../nbs/05_run.ipynb 12
def run_one(
    fname: Union[Path, str],
    lib_name: str = None,
    no_run: bool = False,
    pip_install=False,
    github_issue=False,
    repo=None,
    owner=None,
):
    "Run nb `fname` and timeit, recover exception"
    did_run, skip, exec_time = False, False, time.time()

    # read notebook as dict
    notebook = read_nb(fname)

    # check if notebooks has to be runned
    skip = skip_nb(notebook, lib_name)

    if skip or no_run:
        return "skip", 0
    else:
        did_run, shell = exec_nb(fname, pip_install=pip_install)
    if shell.exc:
        print(shell.prettytb(fname))
        logger.error(f"Error in {fname}:{shell.exc[1]}")
        if github_issue:
            create_issue_nb_fail(fname, shell.prettytb(fname, simple=True), repo=repo, owner=owner)
    return "ok" if did_run else "fail", time.time() - exec_time

# %% ../nbs/05_run.ipynb 14
@call_parse
def run_nbs(
    path: Param("A path to nb files", Path, nargs="?", opt=False) = os.getcwd(),
    verbose: Param("Print errors along the way", store_true) = False,
    lib_name: Param("Python lib names to filter, eg: tensorflow", str) = None,
    no_run: Param("Do not run any notebook", store_true) = False,
    pip_install: Param("Run cells with !pip install", store_true) = False,
    github_issue: Param("Create a github issue if notebook fails", store_true) = False,
    repo: Param("Github repo to create issue in", str) = None,
    owner: Param("Github owner to create issue in", str) = None,
):
    if verbose:
        logger.logger.setLevel(logging.DEBUG)
    path = Path(path)
    files = find_nbs(path)
    logger.info(f"Found {len(files)} notebooks in {path}")
    branch = git_main_name(files[0])

    with Progress(console=logger.console) as progress:
        task_run_nbs = progress.add_task("Running nbs...", total=len(files))
        for fname in files:
            progress.update(task_run_nbs, description=f"Running nb: {str(fname.relative_to(fname.parent.parent))}")
            (run_status, runtime) = run_one(
                fname,
                lib_name=lib_name,
                no_run=no_run,
                pip_install=pip_install,
                github_issue=github_issue,
                repo=repo,
                owner=owner,
            )
            progress.advance(task_run_nbs)
            logger.writerow_incolor(fname, run_status, runtime, colab_link=get_colab_url(fname, branch))
            time.sleep(0.1)

    logger.to_table()
    logger.to_md("run.md")
    return



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_actions.ipynb.

# %% auto 0
__all__ = ['get_colab_url2md', 'create_comment_body', 'get_api', 'upload_modified_nbs', 'open_zip', 'upload_at', 'download',
           'download_payload', 'post_colab_links', 'update_comment', 'create_issue_nb_fail']

# %% ../nbs/03_actions.ipynb 3
import json
from fastcore.all import *
from ghapi.all import *

from .utils import git_main_name, is_nb, git_origin_repo, git_local_repo

# %% ../nbs/03_actions.ipynb 5
def get_colab_url2md(fname: Path, branch="main", github_repo="nb_helpers", as_badge=False) -> str:
    "Create colab links in md"
    colab_url = f"https://colab.research.google.com/github/{github_repo}/blob/{branch}/{str(fname)}"
    if not as_badge:
        return f"[{fname}]({colab_url})"
    return f"[![badge](https://colab.research.google.com/assets/colab-badge.svg)]({colab_url})"

# %% ../nbs/03_actions.ipynb 10
def create_comment_body(title, nb_files, branch, github_repo) -> str:
    "Creates a MD list of fnames with links to colab"
    colab_links = tuple(get_colab_url2md(f, branch, github_repo) for f in nb_files)
    body = tuplify(title) + colab_links
    return "\n -".join(body)

# %% ../nbs/03_actions.ipynb 12
def get_api(owner="wandb", repo="nb_helpers", token=None):
    api = GhApi(owner=owner, repo=repo, token=ifnone(token, github_token()))
    payload = context_github.event
    return api, payload

# %% ../nbs/03_actions.ipynb 14
def upload_modified_nbs(owner="wandb", repo="nb_helpers", token=None, pr_message=None):
    "On PR post a comment with links to open in colab for each changed nb"
    api, payload = get_api(owner, repo, token)

    if "workflow" in payload:
        issue = 1
    else:
        issue = payload.number
    pr = payload.pull_request
    github_repo = pr.head.repo.full_name
    branch = pr.head.ref
    pr_files = [Path(f.filename) for f in api.pulls.list_files(issue)]
    print(f"pr_files: {pr_files}")

    # filter nbs
    nb_files = [f for f in pr_files if is_nb(f)]

    if pr_message is not None:
        with open(pr_message) as pre_title:
            pre_title_md = pre_title.read() + "\n"
    else:
        pre_title_md = ""

    title = pre_title_md + "The following colabs were changed"

    def _get_comment_id(issue):
        comments = api.issues.list_comments(issue)
        candidates = [c for c in comments if title in c.body]
        if len(candidates) == 1:
            comment_id = candidates[0].id
        else:
            comment_id = -1
        return comment_id

    if len(nb_files) > 0:
        body = create_comment_body(title, nb_files, branch, github_repo)
        comment_id = _get_comment_id(issue)
        json.dump( {"pr":issue, 
                    "comment_id":comment_id, 
                    "body": body}, open("modified_colabs.json", 'w' ))

# %% ../nbs/03_actions.ipynb 17
def open_zip(zf, file="reply.json"):
    from zipfile import ZipFile
    from io import BytesIO

    zip_file = ZipFile(BytesIO(zf))
    f = zip_file.open(file)
    return f

def upload_at(owner="wandb", repo="nb_helpers", token=None):
    api, payload = get_api(owner, repo, token)
    print("Uploading file to artifact\n=============")
    json.dump( {0: "My dummy issue reply"}, open("reply.json", 'w' ))
    
def download(owner="wandb", repo="nb_helpers", token=None):
    api, payload = get_api(owner, repo, token)
    print("HERE IS THE PAYLOAD\n=============")
    print(payload)
    print("=============\n\n")
    at = api.actions.list_workflow_run_artifacts(payload.workflow_run.id)["artifacts"][0]
    print("HERE IS THE Artifact\n=============")
    print(at)
    print("=============\n\n")
    at_id = at["id"]
    print("=============\n\n")
    print(f"Downloading AT {at_id}\n")
    zf = api.actions.download_artifact(at_id, "zip")
    print(f"Extracting reply.json: {at_id}")
    unzipped_file = open_zip(zf)
    print(unzipped_file, type(unzipped_file))
    line = unzipped_file.readline()
    f_dict = json.loads(line)
    print(f"Content of reply.json: {f_dict}")
    print("Finished")

# %% ../nbs/03_actions.ipynb 22
def download_payload(owner="wandb", repo="nb_helpers", token=None):
    "Get artifact with text and create comment"
    api, payload = get_api(owner, repo, token)
    print("HERE IS THE PAYLOAD\n=============")
    print(payload)
    print("=============\n\n")
    at = api.actions.list_workflow_run_artifacts(payload.workflow_run.id)["artifacts"][0]
    print("HERE IS THE Artifact\n=============")
    print(at)
    print("=============\n\n")
    at_id = at["id"]
    print("=============\n\n")
    print(f"Downloading AT {at_id}\n")
    zf = api.actions.download_artifact(at_id, "zip")
    print(f"Extracting reply.json\n")
    f = open_zip(zf, file="modified_colabs.json")
    return json.loads(f.readline())

# %% ../nbs/03_actions.ipynb 24
def post_colab_links(owner="wandb", repo="nb_helpers", token=None):
    api, _ = get_api(owner, repo, token)
    message = download_payload(owner, repo, token)
    pr, comment_id, body = message["pr"], message["comment_id"], message["body"]
    if comment_id > 0:
        print(f">> Updating comment on PR #{pr}\n{body}\n")
        api.issues.update_comment(comment_id, body)
    else:
        print(f">> Creating comment on PR #{pr}\n{body}\n")
        api.issues.create_comment(issue_number=pr, body=body)

# %% ../nbs/03_actions.ipynb 25
def update_comment(issue, body, comment_id, owner="wandb", repo="nb_helpers", token=None):
    api, payload = get_api(owner, repo, token)
    
    if "workflow" in payload:
        issue = 1
    else:
        issue = payload.number
        
    if comment_id > 0:
        print(f">> Updating comment on PR #{issue}\n{body}\n")
        api.issues.update_comment(comment_id, body)
    else:
        print(f">> Creating comment on PR #{issue}\n{body}\n")
        api.issues.create_comment(issue_number=issue, body=body)

# %% ../nbs/03_actions.ipynb 27
def create_issue_nb_fail(fname, traceback=None, owner="wandb", repo="nb_helpers", token=None):
    "Creates issue of failing nb"
    print("="*75)
    api, _ = get_api(owner, repo, token)
    fname = fname.resolve()
    github_repo = git_origin_repo(fname)
    title = f"Failed to run {fname}"
    branch = git_main_name(fname)
    fname = fname.relative_to(git_local_repo(fname))
    repo_url = f"[{str(fname)}](https://github.com/{github_repo}/blob/{branch}/{str(fname)})"
    colab_url = get_colab_url2md(fname, branch, github_repo, as_badge=True)
    traceback = ifnone(traceback, "")
    body = (
        "The following notebooks failed to run:\n\n"
        "|notebook name|               |\n"
        "|-------------|---------------|\n"
        f"| {repo_url}   | {colab_url} |\n\n"
        "------------------------------\n"
        "The recovered traceback is:\n\n"
        "```python\n"
        f"{traceback}\n"
        "```"
    )
    api.issues.create(title=title, body=body, labels=["bug"])



import os, wandb, docker
from pathlib import Path

DOCKER_IMAGE = "lambda-meta:latest"

WANDB_API_KEY=wandb.api.api_key
GITHUB_TOKEN=os.environ["GITHUB_TOKEN"]

ENVS = {"WANDB_API_KEY":WANDB_API_KEY, 
        "GITHUB_TOKEN":GITHUB_TOKEN}

client = docker.from_env()
device_requests=[docker.types.DeviceRequest(device_ids=["0"], 
                                            capabilities=[['gpu']])]

examples_path = Path("/home/tcapelle/wandb/examples")
examples_path_in_docker = Path("/examples")
working_dir = Path('/home/examples')
volumes = {str(examples_path): {'bind': str(working_dir), 'mode': 'rw'}}

def in_docker_path(nb, examples_dir):
    return examples_dir / nb.relative_to(examples_path)

def run_one (nb, docker_image, envs, gpu=True, github_issue=True, from_local=True, pip_install=True):
    nb = in_docker_path(nb, working_dir if from_local else examples_path_in_docker)
    pip_install = "--pip-install" if pip_install else ""
    github_issue = "--github_issue --repo examples_ci --owner wandb" if github_issue else ""
    cmd = f"nb_helpers.run_nbs {nb}"+f" {pip_install}"+f" {github_issue}"
    res = client.containers.run(
        image=docker_image, 
        command=cmd, 
        environment=envs, 
        remove=True,
        volumes=volumes,
        device_requests=device_requests if gpu else None,
        )
    return res


from pkg_resources import parse_version
from configparser import ConfigParser
import setuptools, re, sys

assert parse_version(setuptools.__version__) >= parse_version("36.2")

# note: all settings are in settings.ini; edit there, not here
config = ConfigParser(delimiters=["="])
config.read("settings.ini")
cfg = config["DEFAULT"]

cfg_keys = "version description keywords author author_email".split()
expected = cfg_keys + "lib_name user branch license status min_python audience language".split()
for o in expected:
    assert o in cfg, "missing expected setting: {}".format(o)
setup_cfg = {o: cfg[o] for o in cfg_keys}

if len(sys.argv) > 1 and sys.argv[1] == "version":
    print(setup_cfg["version"])
    exit()

licenses = {
    "apache2": ("Apache Software License 2.0", "OSI Approved :: Apache Software License"),
    "mit": ("MIT License", "OSI Approved :: MIT License"),
}
statuses = [
    "1 - Planning",
    "2 - Pre-Alpha",
    "3 - Alpha",
    "4 - Beta",
    "5 - Production/Stable",
    "6 - Mature",
    "7 - Inactive",
]
py_versions = "3.6 3.7 3.8 3.9 3.10".split()
min_python = cfg["min_python"]
lic = licenses[cfg["license"]]

requirements = ["pip", "packaging"]
if cfg.get("requirements"):
    requirements += cfg.get("requirements", "").split()
if cfg.get("pip_requirements"):
    requirements += cfg.get("pip_requirements", "").split()
dev_requirements = (cfg.get("dev_requirements") or "").split()

long_description = open("README.md").read()

setuptools.setup(
    name=cfg["lib_name"],
    license=lic[0],
    classifiers=[
        "Development Status :: " + statuses[int(cfg["status"])],
        "Intended Audience :: " + cfg["audience"].title(),
        "License :: " + lic[1],
        "Natural Language :: " + cfg["language"].title(),
    ]
    + ["Programming Language :: Python :: " + o for o in py_versions[py_versions.index(min_python) :]],
    url=cfg["git_url"],
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=requirements,
    extras_require={"dev": dev_requirements},
    python_requires=">=" + cfg["min_python"],
    long_description=long_description,
    long_description_content_type="text/markdown",
    zip_safe=False,
    entry_points={"console_scripts": cfg.get("console_scripts", "").split()},
    **setup_cfg
)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/04_clean.ipynb.

# %% auto 0
__all__ = ['CONSOLE', 'CLEAN_TABLE', 'colab_json', 'cell_metadata_keep', 'nb_metadata_keep', 'rm_execution_count',
           'clean_output_data_vnd', 'clean_cell_output', 'clean_cell', 'clean_nb', 'clean_one', 'clean_all',
           'clean_nbs']

# %% ../nbs/04_clean.ipynb 3
import io, json, os
from pathlib import Path

from rich import print
from rich.console import Console
from rich.table import Table
from rich.progress import track
from fastcore.script import call_parse, Param, store_true
from execnb.nbio import read_nb

from .utils import print_output, is_nb, find_nbs

# %% ../nbs/04_clean.ipynb 5
def _create_table():
    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("Notebook Path", style="dim")
    table.add_column("Status")
    return table

# %% ../nbs/04_clean.ipynb 6
CONSOLE = Console()

# %% ../nbs/04_clean.ipynb 7
CLEAN_TABLE = _create_table()

# %% ../nbs/04_clean.ipynb 9
def rm_execution_count(o):
    "Remove execution count in `o`"
    if "execution_count" in o:
        o["execution_count"] = None

# %% ../nbs/04_clean.ipynb 10
colab_json = "application/vnd.google.colaboratory.intrinsic+json"

# %% ../nbs/04_clean.ipynb 11
def clean_output_data_vnd(o):
    "Remove `application/vnd.google.colaboratory.intrinsic+json` in data entries"
    if "data" in o:
        data = o["data"]
        if colab_json in data:
            new_data = {k: v for k, v in data.items() if k != colab_json}
            o["data"] = new_data

# %% ../nbs/04_clean.ipynb 12
def clean_cell_output(cell):
    "Remove execution count in `cell`"
    if "outputs" in cell:
        for o in cell["outputs"]:
            rm_execution_count(o)
            clean_output_data_vnd(o)
            o.get("metadata", o).pop("tags", None)

# %% ../nbs/04_clean.ipynb 13
cell_metadata_keep = ["hide_input"]

# %% ../nbs/04_clean.ipynb 14
nb_metadata_keep = ["kernelspec", "accelerator", "colab"]

# %% ../nbs/04_clean.ipynb 15
def clean_cell(cell, clear_all=False):
    "Clean `cell` by removing superfluous metadata or everything except the input if `clear_all`"
    rm_execution_count(cell)
    if "outputs" in cell:
        if clear_all:
            cell["outputs"] = []
        else:
            clean_cell_output(cell)
    if cell["source"] == [""]:
        cell["source"] = []
    cell["metadata"] = {} if clear_all else {k: v for k, v in cell["metadata"].items() if k in cell_metadata_keep}

# %% ../nbs/04_clean.ipynb 16
def clean_nb(nb, clear_all=False):
    "Clean `nb` from superfluous metadata, passing `clear_all` to `clean_cell`"
    for c in nb["cells"]:
        clean_cell(c, clear_all=clear_all)
    nb["metadata"] = {k: v for k, v in nb["metadata"].items() if k in nb_metadata_keep}

# %% ../nbs/04_clean.ipynb 18
def clean_one(fname: Path, clear_outs: bool = False, disp: bool = False):
    """Clean notebook metadata:
    - `clear_all` removes also outputs
    - `disp` prints to stdout
    """
    if not is_nb(fname):
        print(f"This {fname}: is not a notebook my friend")
        return
    notebook = json.load(open(str(fname), "r", encoding="utf-8"))
    clean_nb(notebook, clear_all=clear_outs)
    if disp:
        print_output(notebook)
    else:
        x = json.dumps(notebook, sort_keys=True, indent=1, ensure_ascii=False)
        with io.open(fname, "w", encoding="utf-8") as f:
            f.write(x)
            f.write("\n")

# %% ../nbs/04_clean.ipynb 20
def clean_all(path: Path, clear_outs=True, disp=False):
    "Apply clean to all nbs inside path recursvely"

    for nb in track(find_nbs(path), "Cleaning nbs..."):
        try:
            clean_one(nb, clear_outs, disp)
            CLEAN_TABLE.add_row(str(nb), "[green]Ok[/green]:heavy_check_mark:")
        except:
            CLEAN_TABLE.add_row(str(nb), "[red]Failed[/red]")

# %% ../nbs/04_clean.ipynb 22
@call_parse
def clean_nbs(
    path: Param("A path to nb files", Path, nargs="?", opt=False) = os.getcwd(),
    clear_outs: Param("Remove cell outputs", store_true) = False,
    verbose: Param("Rnun on verbose mdoe", store_true) = False,
):
    "Clean notebooks on `path` from useless metadata"
    path = Path(path)
    clean_all(path, clear_outs, disp=verbose)
    CONSOLE.print(CLEAN_TABLE)
